@@ -2,7 +2,7 @@ package Test2;
 use strict;
 use warnings;
 
-our $VERSION = '0.000022';
+our $VERSION = '0.000023';
 
 1;
 
@@ -28,7 +28,7 @@ completely refactoring it, adding many new features and capabilities.
 =head1 GETTING STARTED
 
 If you are interested in writing tests using new tools then you should look at
-B<NOT YET DETERMINED>.
+L<Test2::Suite>.
 
 If you are interested in writing new tools you should take a look at
 L<Test2::API> first.
