@@ -16,6 +16,7 @@ use Test2::Util::HashBase qw{
     _listeners
     _follow_ups
     _formatter
+    _context_aquire
     _context_init
     _context_release
 
@@ -179,6 +180,24 @@ sub follow_up {
     push @{$self->{+_FOLLOW_UPS}} => $sub;
 }
 
+sub add_context_aquire {
+    my $self = shift;
+    my ($sub) = @_;
+
+    croak "add_context_aquire only takes coderefs for arguments, got '$sub'"
+        unless ref $sub && ref $sub eq 'CODE';
+
+    push @{$self->{+_CONTEXT_AQUIRE}} => $sub;
+
+    $sub; # Intentional return.
+}
+
+sub remove_context_aquire {
+    my $self = shift;
+    my %subs = map {$_ => $_} @_;
+    @{$self->{+_CONTEXT_AQUIRE}} = grep { !$subs{$_} == $_ } @{$self->{+_CONTEXT_AQUIRE}};
+}
+
 sub add_context_init {
     my $self = shift;
     my ($sub) = @_;
@@ -613,6 +632,31 @@ only argument to your codeblock will be a L<Test2::Util::Trace> instance.
 follow_up subs are called only once, ether when done_testing is called, or in
 an END block.
 
+=item $sub = $hub->add_context_aquire(sub { ... });
+
+Add a callback that will be called every time someone tries to aquire a
+context. It gets a single argument, a reference the the hash of parameters
+being used the construct the context. This is your chance to change the
+parameters by directly altering the hash.
+
+    test2_add_callback_context_aquire(sub {
+        my $params = shift;
+        $params->{level}++;
+    });
+
+This is a very scary API function. Please do not use this unless you need to.
+This is here for L<Test::Builder> and backwards compatability. This has you
+directly manipulate the hash instead of returning a new one for performance
+reasons.
+
+B<Note> Using this hook could have a huge performance impact.
+
+The coderef you provide is returned and can be used to remove the hook later.
+
+=item $hub->remove_context_aquire($sub);
+
+This can be used to remove a context aquire hook.
+
 =item $sub = $hub->add_context_init(sub { ... });
 
 This allows you to add callbacks that will trigger every time a new context is
